from django.apps import AppConfig

class WagtailWordpressImportImportersConfig(AppConfig):
    label = "wagtail_wordpress_import_importers"
    name = "wagtail_wordpress_import_importers"
    verbose_name = "Wagtail wordpress import importers"
