# Generated by Django 3.2.7 on 2021-11-02 17:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0002_add_categories'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='name',
            field=models.CharField(max_length=255, unique=True),
        ),
    ]
