# Generated by Django 3.2.7 on 2021-11-16 10:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0004_alter_body_stream_feild'),
    ]

    operations = [
        migrations.AddField(
            model_name='testpage',
            name='wp_post_meta',
            field=models.JSONField(blank=True, null=True),
        ),
    ]
