from django.apps import AppConfig

class WagtailWordpressImportAppConfig(AppConfig):
    label = "wagtail_wordpress_import"
    name = "wagtail_wordpress_import"
    verbose_name = "Wagtail wordpress import"
