from django.apps import AppConfig


class WagtailWordpressImportTestAppConfig(AppConfig):
    label = "wagtail_wordpress_import_test"
    name = "wagtail_wordpress_import.test"
    verbose_name = "Wagtail wordpress import tests"
