{% load wagtailimages_tags %}

{% if value.link %}
<a href="{{ value.link }}">
    <figure class="align-{{ value.alignment }}">
        {% image value.image width-800 %}
        {% if value.caption %}
        <figcaption>{{ value.caption }}</figcaption>
        {% endif %}
    </figure>
</a>
{% else %}
<figure class="align-{{ value.alignment }}">
    {% image value.image width-800 %}
    {% if value.caption %}
    <figcaption>{{ value.caption }}</figcaption>
    {% endif %}
</figure>
{% endif %}
